/*****************  Standard C Source Header ****************
*
* PLEASE SET YOUR EDITOR FOR 4 TABS
*
*
* **********************************************************************************
*
* GENERAL INFORMATION
*
*   Module Description:     Serial LCD driver library for PICVUE-064 with AVR controller
                            Serial LCD Library for PICVUE PC-064-PYC 2x8 Character LCD module
                            Uses HD44780 compatible controller.

        This library can be used with any LCD character module using a HD44780 controller
        and two each 74HC164 shift registers.
*
*
*   File Name........... SER_LCD.C  - LCD support functions
*
*   VERSION NO.......... 1.01
*   PRIMARY AUTHOR ..... GJO
*   OTHER AUTHORS ...... GJO
*   START DATE ......... Feb 24, 2005
*   RELEASE DATE........
*   JOB NUMBER.......... n/a
*   PROJECT NAME........ n/a
*   PRODUCT NAME........ n/a
*   PRODUCT NUMBER...... n/a
*   COMMENTS............
*
* FORMAL DOCUMENTATION
*
*   PROGRAMMER'S MANUAL ........... None
*   NAME OF FILE CONTAINED IN...... n/a
*   COMMENTS....................... In source file only
*   USER'S MANUAL ................. None
*
*   LIST ANY OTHER REFERENCE DOCUMENTS
*
*   Dependencies:   ser_LCD_V101.h, fonts.h
*
*   Revision list:
*
*   rev	    date		who		description
*   ----	-------		---		------------------------------------------------
*   1.00	05.02.26	GJO		Wrote library and tested functionality 
*   1.01    05.03.09    GJO     Added custom graphics loader
*                                                                     
    LCD User Functions included:
 ===============================================================================
    
    line(char which_line);                  // Set current line
    clear_display(void);                    // Clear LCD screen

    set_LCD_cur(LCDrow, LCDcol);            // Set new row and cursor position

    wr_disp_ser(data)                       // Low level LCD write function

    disp_char(c);                           // Display single ASCII character 
    disp_str(*sa);                          // Display string by RAM pointer
    disp_cstr(*sa);                         // Display string by ROM pointer
    
    LCD_CursorLeft ();                      // Move cursor left
    LCD_CursorRight ();                     // Move Cursor Right
    LCD_CursorOn ();                        // Turn Cursor On
    LCD_CursorOff ();                       // Turn Cursor Off
    LCD_DisplayOff ();                      // Turn Display Off
    LCD_DisplayOn ();                       // Turn Display On
    
 
    
*/

/******************************************************************************/
//************************* LCD DISPLAY FUNCTIONS *****************************
/******************************************************************************/

/******************************************************************************
* Function:      
* Description:  LCD_CursorLeft: Move the cursor left by one character.
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void LCD_CursorLeft (void)
{
  glrsel = 0;
  wr_disp_ser(0x10);
}

/******************************************************************************
* Function:     
* Description:  LCD_CursorRight: Move the cursor right by one character.
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void LCD_CursorRight (void)
{
  glrsel = 0;
  wr_disp_ser(0x14);
}

/******************************************************************************
* Function:     
* Description:  LCD_CursorOn: Turn the cursor on.
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void LCD_CursorOn (void)
{
  glrsel = 0;
  wr_disp_ser(0x0d);
}

/******************************************************************************
* Function:     
* Description:  LCD_CursorOff: Turn the cursor off.
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void LCD_CursorOff (void)
{
  glrsel = 0;
  wr_disp_ser(0x0c);
}

/******************************************************************************
* Function:     
* Description:  LCD_DisplayOff: Turn Off LCD.
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void LCD_DisplayOff (void)
{
  glrsel = 0;
  wr_disp_ser(0x08);
}

/******************************************************************************
* Function:     
* Description:  LCD_DisplayOn: Turn On LCD.
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void LCD_DisplayOn (void)
{
  glrsel = 0;
  wr_disp_ser(0x0c);
}

/******************************************************************************
* Function:     write_custom_chars() 
* Description:  Write a user defined characters to LCD
* Dependencies: fonts.h
*               ser_lcd.h
* Note:         none
* Returns:      none
*******************************************************************************/
void write_custom_chars_LCD(void)
{
    LCD_DefineChar (0, LCD_CHAR_UP_ARROW);
    LCD_DefineChar (1, LCD_CHAR_DOWN_ARROW);
    LCD_DefineChar (2, LCD_CHAR_TRADEMARK_T);
    LCD_DefineChar (3, LCD_CHAR_TRADEMARK_M);
    LCD_DefineChar (4, LCD_CHAR_BAR1);
    LCD_DefineChar (5, LCD_CHAR_BAR2);
    LCD_DefineChar (6, LCD_CHAR_BAR3);
    LCD_DefineChar (7, LCD_CHAR_BAR4);
}

/******************************************************************************
* Function:     void LCD_DefineChar (char address, const unsigned char *pattern) 
* Description:  Define dot pattern for user-defined character.
* Dependencies: fonts.h
* inputs: address = address of character (0x00-0x07)
*         pattern = pointer to 8-byte array containing the dot pattern
* Note:         
* Returns:      none
*******************************************************************************/
void LCD_DefineChar (char address, char flash *pattern)
{
  char i;
  
  glrsel = 0;                           // switch LCD to command mode
  wr_disp_ser(0x40 + (address << 3));   // Set LCD data pointer to begin of CGRAM
  delay_ms(2);    
  
  glrsel = 0xFF;                        // switch LCD to data mode
      
  for (i=0; i<8; i++)
  {
     wr_disp_ser(*pattern++);           // write eight data bytes to LCD CGRAM data buffer
     delay_ms(2);      
  }
}

/******************************************************************************
* Function:     wr_disp_ser (unsigned char data)
* Description:  Write two bytes to the SPI LCD display, folowed by E pulse:
* 
*    SCK _____ (8 clks)   ______            ________________
*             \/\/\/\/\/\/      \/\/\/\/\/\/
*              __________        __________
*    MOSI_____/ RS SEL   \______/ LCD data \________________ SPI data Xfer
*             \__________/      \__________/
*            MSB (0x80)         MSB        LSB
*             |-- 10us --|      |---10us---|     ___
*    PC.7  _____________________________________/   \_______ E-strobe to update HD44780
*

* Note:         Code written for 74HC164 16 bit serial transfer into U2/U3
* Returns:      none
*******************************************************************************/
void wr_disp_ser(char data)
{
    LCD_CLK = 0;                    // Activate E clk
    SPDR = glrsel;                  // Load SPI data register with LCD command byte
	delay_us(10);                   // Allow time to shift data out
    SPDR = data;                    // Load SPI data register with LCD data byte
	delay_us(10);                   // Wait again for finishing the shifting
    LCD_CLK = 1;                    // Activate E clk
    delay_us(5);                    // allow display time to accept the new data
    LCD_CLK = 0;                    // Activate E clk
    delay_ms(2);
}

/******************************************************************************
* Function:     init_disp_ser(void)
* Description:  Setup SPI LCD display for application
* Returns:      none
*******************************************************************************/
void init_display_ser(void)
{
    //char i,j;
    glrsel = 0;                         // command mode
    delay_ms(50);                       // allow display time to accept the new data

    wr_disp_ser(0x38);                  // Initialize LCD controller Begin
    delay_ms(5); 
    wr_disp_ser(0x38);                  // Repeat
    delay_ms(5); 
    wr_disp_ser(0x38);                  // Repeat
    delay_ms(5); 
    wr_disp_ser(0x08);                  // set cursor to move right, no shifting of LCD display
    delay_ms(2); 
    wr_disp_ser(0x01);                  // Clear Display memory
    delay_ms(2); 
    wr_disp_ser(0x10);                  // Set cursor to move
    delay_ms(2); 
    wr_disp_ser(0x06);                  // turn display on, cursor off, cursor blinking off
    delay_ms(2); 
    wr_disp_ser(0x0c);                  // 
    delay_ms(2); 
}

/******************************************************************************
* Function:     void line(char which_line)
* Description:  Move to line specified by which_line
* Uses:         which_line to selct LCD line to be used
* Returns:      none
*******************************************************************************/
void line(char which_line)
{
    glrsel = 0;
    LCD_ADDR = addLUT[which_line - 1];
    wr_disp_ser(LCD_ADDR);              // move to which_line
    glrsel = 0xFF;                      // Switch to data mode
    LCD_ADDR = 0;
    LCD_LINE = which_line;
}

/******************************************************************************
* Function:     void clear_display(void)
* Description:  Clear LCD
* Uses:         none
* Returns:      none
*******************************************************************************/
void clear_display(void)
{
    glrsel = 0;
    wr_disp_ser(0x01);          // Clear display and home cursor
    delay_ms(10);
    glrsel = 0xFF;
    line(1);
}

/******************************************************************************
* Function:     void set_LCD_cur(char LCDrow, char LCDcol)
* Description:  Set row, col cursor position in LCD
* Uses:         LCDrow, LCDcol
* Returns:      none
*******************************************************************************/
void set_LCD_cur(char LCDrow, char LCDcol)
{
    glrsel = 0;
    LCD_ADDR = addLUT[LCDrow] + LCDcol;
    wr_disp_ser(LCD_ADDR);          // move to row and column
    glrsel = 0xFF;
    LCD_LINE = LCDrow;
}

/******************************************************************************
* Function:     void disp_char(unsigned char c)
* Description:  display a character on LCD
* Uses:         character c, gl LCD_LINE, LCD_ADDR
* Returns:      none
*******************************************************************************/
void disp_char(unsigned char c)
{
    glrsel = 0xFF;
    wr_disp_ser(c);
    LCD_ADDR++;
    // Remember to add RAM offset table value to test
    if (LCD_ADDR == (addLUT[LCD_LINE] + 8) )          // automatically wrap next line
    {
        line(2);
    }
}

/******************************************************************************
* Function:     void disp_str(unsigned char *sa)
* Description:  display a string from RAM
* Uses:         pointer to string in RAM
* Returns:      none
*******************************************************************************/
void disp_str(unsigned char *sa)
{
    while (*sa != 0)
        disp_char(*sa++);
}

/******************************************************************************
* Function:     void disp_cstr(unsigned char *sa)
* Description:  display a string from ROM
* Uses:         Pointer to string in ROM
* Returns:      none
*******************************************************************************/
void disp_cstr(unsigned char flash *sa)
{
    while (*sa != 0)
        disp_char(*sa++);
}

/******************************************************************************
* Function:     void init_SPI()
* Description:  display a string from ROM
* Uses:         Pointer to string in ROM
* Returns:      none
*******************************************************************************/
void init_SPI()
{

// SPI Initialization
	// B7 SPIE = 0 - SPI Interrupt enable
	// B6 SPE  = 1 - SPI Enable Bit
	// B5 DORD = 1 - Data Order Bit: LSB first sent if set
	// B4 MSTR = 1 - Master = 1
	// B3 CPOL = 0 - Clock Polarity, 0 = pos, 1 = neg
	// B2 CPHA = 0 - Clock Phase,    0 = rising edge sample, 1 = falling edge
	// B1 SPR1 = 0 - Clock Rate Select
	// B0 SPR0 = 0 - Clock Rate Select
	SPCR = 0b01110000;    // Correct Setting for U2/U3

}

// End of file
