/*****************  Standard C Source Header ****************
*
* PLEASE SET YOUR EDITOR FOR 4 TABS
*
*
* **********************************************************************************
*
* GENERAL INFORMATION
*
*   Module Description:     Serial LCD driver library for PICVUE-064 with AVR controller
                            Serial LCD Library for PICVUE PC-064-PYC 2x8 Character LCD module
                            Uses HD44780 compatible controller.

        This library can be used with any LCD character module using a HD44780 controller
        and two each 74HC164 shift registers.
*
*
*   FILE NAME........... SER_LCD.H  - This file
*   File Name........... SER_LCD.C  - LCD support functions
*
*   VERSION NO.......... 1.00
*   PRIMARY AUTHOR ..... GJO
*   OTHER AUTHORS ...... GJO
*   START DATE ......... Feb 24, 2005
*   RELEASE DATE........
*   JOB NUMBER.......... n/a
*   PROJECT NAME........ n/a
*   PRODUCT NAME........ n/a
*   PRODUCT NUMBER...... n/a
*   COMMENTS............
*
* FORMAL DOCUMENTATION
*
*   PROGRAMMER'S MANUAL ........... None
*   NAME OF FILE CONTAINED IN...... n/a
*   COMMENTS....................... In source file only
*   USER'S MANUAL ................. None
*
*   LIST ANY OTHER REFERENCE DOCUMENTS
*
*   Custom graphics code based on PC examples by Randy Rasa:
*   * Copyright 1996 by Randy Rasa
*   *                   rrasa@sky.net
*   *                   http://www.sky.net/~rrasa
*
*   Revision list:
*
*   rev	    date		who		description
*   ----	-------		---		------------------------------------------------
*   1.00	05.02.26	GJO		Wrote library and tested functionality
*   1.01    05.03.09    GJO     Added custom graphics loader based on Randy's example
*
*
*   ** Remember to update the version string! **
*
* ENVIRONMENT INFORMATION
*   Development Platform .......... Pentium III 1215 MHz, 512MB RAM, Win2K
*   Development IDE ............... CodeVision AVR 1.24.5
*   Compiler Used ................. CodeVision AVR 1.24.5
*	Compile Options................ n/a
*   Libraries Used ................ SER_LCD.H
*   Target OS Platform ............ No OS
*	Target Processor .............. ATMEL ATMEGA32 and most other AVRs
*   Hardware Platforms Tested on .. AVRSIM100 + DT003 Motherboard with SHT71 Sensor attached
*   Programmer Used................ AVR ISP Serial Programmer from Digi-Key
*
* REFERENCES
*       1) HD44780U.PDF Hitachi LCD data sheet
*       2) ATMEGA32 data sheet
*       3) LCD data sheet for HD44780, PICVUE PC-064-PYH
*
*
* Serial 2x8 LCD module (Uses HC164 Shift register feeding a HD44780)
* DT003 wiring interface used for prototype testing:
*-----------------------------------------------------------------------
*   Pin     Function            SIM100 Bus          LCD pin layout
*-----------------------------------------------------------------------
*   1       SPI CLK             D7  22       |  Top     o  | CLK
*   2       SPI DATA            D5  20       |  of      o  | DATA
*   3       not used                         |  LCD        | no pin
*   4       GND                     9        |          o  | GND
*   5       E CLK               D4  19       |          o  | E
*   6       +5V                     7        |          o  | +5V
*-----------------------------------------------------------------------

    LCD data interface information:

    The data sent to the display is in the format:
    (This is handled by disp_byte_ser(char)automatically)

    See init_SPI() for further information

    SCK _____ (8 clks)   ______            ________________
             \/\/\/\/\/\/      \/\/\/\/\/\/
              __________        __________
    MOSI_____/ RS SEL   \______/ LCD data \________________ SPI data Xfer
             \__________/      \__________/
            MSB (0x80)         MSB        LSB
             |-- 10us --|      |---10us---|     ___
    PC.7  _____________________________________/   \_______ E-strobe to update HD44780
                                         5us
    SPI configuration:  Positive clock polarity
                        LSB sent first
                        Clock Polarity normal
                        Clock speed 1us

    Example of use:

    init_SPI();                 // Initialize AVR SPI hardware
    init_display_ser();         // Initialize HD44780 control registers
    clear_display();            // Clear LCD RAM

	set_LCD_cur(0,0);           // Set LCD cursor
	disp_cstr("Line1");         // Display some text
    set_LCD_cur(1,0);           // Set LCD cursor to next line
	disp_cstr("Line2");         // Display some more text

******************************************************************************/

// LCD line offset addresses
const unsigned char flash addLUT[2] = {0x80,0xC0};

// Used to remember current row and column position
unsigned char LCD_ADDR, LCD_LINE;

char glrsel = 0;  // SPI LCD register select mode

// LCD HD44780 E strobe clock pin
#define LCD_CLK             PORTC.7         // Change to suit design hardware

// LCD functions prototypes

void line(char which_line);                 // Set cuurent line
void clear_display(void);                   // Clear LCD screen

void set_LCD_cur(char LCDrow, char LCDcol); // Set new row and cursor position

void disp_char(char c);                     // Display single ASCII character at current position
void disp_str(char *sa);                    // Display string referenced by RAM pointer
void disp_cstr(char flash *sa);             // Display string referenced to ROM literal text

void wr_disp_ser(char data);                // Low level LCD byte write

void init_display_ser(void);                // Initializes HD44780 registers
void init_SPI();                            // Initializes AVR SPI hardware

// These are based on Randy Rasa's code examples ported from GCCAVR to CV:

void LCD_DefineChar (char address, char flash *pattern);    // Load fonts into CGRAM
void write_custom_chars_LCD(void);

void LCD_CursorLeft (void);
void LCD_CursorRight (void);
void LCD_CursorOn (void);
void LCD_CursorOff (void);
void LCD_DisplayOff (void);
void LCD_DisplayOn (void);


